<?php

namespace App\Controller;

use App\Entity\Printer;
use App\Enum\ErrorMessages;
use App\Repository\PrinterRepository;
use App\Service\PrinterService;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

class PrinterCrudController extends AbstractController
{
    public function __construct(
        private readonly PrinterRepository $printerRepository,
        private readonly PrinterService    $printerService,
    )
    {
    }

    #[Route('/printer', name: 'printer', methods: ['GET'])]
    public function getAllPrinters(): JsonResponse
    {
        return $this->json($this->printerRepository->findAll());
    }

    #[Route('/printer/{id}', name: 'get_printer', methods: ['GET'])]
    public function getPrinter(?Printer $printer): JsonResponse
    {
        //TODO: ungleich null und in funktion auslagern
        if (!$printer) {
            return $this->json([
                'message' => ErrorMessages::DOESNT_EXIST->value,
            ], Response::HTTP_NOT_FOUND);
        }

        return $this->json($printer);
    }

    #[Route('/printer/{id}', name: 'delete_printer', methods: ['DELETE'])]
    public function deletePrinter(?Printer $printer): JsonResponse
    {
        if (!$printer) {
            return $this->json([
                'message' => ErrorMessages::DOESNT_EXIST->value,
            ], Response::HTTP_NOT_FOUND);
        }

        $this->printerService->deletePrinter($printer);

        return $this->json([
            'message' => 'printer was deleted',
        ]);
    }

    #[Route('/printer', name: 'create_printer', methods: ['POST'])]
    public function createPrinter(Request $request): JsonResponse
    {
        $jsonContent = $request->getContent();
        if (!$this->printerService->validateJson($jsonContent)) {
            return $this->json([
                'message' => ErrorMessages::DATA_INCOMPLETE->value,
            ], Response::HTTP_NOT_FOUND);
        }

        $printer = $this->printerService->createPrinter($jsonContent);

        return $this->json($printer);
    }

    #[Route('/printer/{id}', name: 'edit_printer', methods: ['PUT'])]
    public function editPrinter(?Printer $printer, Request $request): JsonResponse
    {
        if (!$printer) {
            return $this->json([
                'message' => ErrorMessages::DOESNT_EXIST->value,
            ], Response::HTTP_NOT_FOUND);
        }

        $this->printerService->editPrinter($printer, $request->getContent());

        return $this->json($printer);
    }
}
